from .provider.voe import get_direct_link_from_voe
from .provider.vidoza import get_direct_link_from_vidoza
from .provider.vidmoly import get_direct_link_from_vidmoly
from .provider.doodstream import get_direct_link_from_doodstream
from .provider.speedfiles import get_direct_link_from_speedfiles
from .provider.luluvdo import get_direct_link_from_luluvdo
from .provider.hanime import get_direct_link_from_hanime
