from .common import (
    get_github_release,
    download_mpv,
    download_syncplay,
    download_file,
    get_season_episode_count,
    get_movie_episode_count,
    generate_links,
    remove_anime4k,
    remove_mpv_scripts,
    copy_file_if_different,
    setup_autostart,
    setup_autoexit
)
